@@ -1,23 +1,22 @@
 /*
 	common: misc stuff... audio flush, status display...
 
-	copyright ?-2007 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 */
 
 #include "mpg123app.h"
 #include <sys/stat.h>
-#include <sys/time.h>
-
 #include <fcntl.h>
-
 #include "common.h"
 
 #ifdef WIN32
 #include <winsock.h>
 #endif
 
+#include "debug.h"
+
 const char* rva_name[3] = { "off", "mix", "album" };
 static const char *modes[5] = {"Stereo", "Joint-Stereo", "Dual-Channel", "Single-Channel", "Invalid" };
 static const char *smodes[5] = { "stereo", "joint-stereo", "dual-channel", "mono", "invalid" };
