@@ -1,14 +1,14 @@
 /*
 	control_generic.c: control interface for frontends and real console warriors
 
-	copyright 1997-99,2004-6 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1997-99,2004-8 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Andreas Neuhaus and Michael Hipp
 	reworked by Thomas Orgis - it was the entry point for eventually becoming maintainer...
 */
 
+#include "mpg123app.h"
 #include <stdarg.h>
-#include <sys/time.h>
 #ifndef WIN32
 #include <sys/wait.h>
 #include <sys/socket.h>
@@ -19,7 +19,6 @@
 #include <errno.h>
 #include <string.h>
 
-#include "mpg123app.h"
 #include "common.h"
 #include "buffer.h"
 #include "genre.h"
@@ -38,6 +37,8 @@ FILE *outstream;
 static int mode = MODE_STOPPED;
 static int init = 0;
 
+#include "debug.h"
+
 void generic_sendmsg (const char *fmt, ...)
 {
 	va_list ap;
@@ -433,7 +434,7 @@ int control_generic (mpg123_handle *fr)
 					generic_sendmsg("H {");
 					generic_sendmsg("H HELP/H: command listing (LONG/SHORT forms), command case insensitve");
 					generic_sendmsg("H LOAD/L <trackname>: load and start playing resource <trackname>");
-					generic_sendmsg("H LOADPAUSED/LP <trackname>: load and start playing resource <trackname>");
+					generic_sendmsg("H LOADPAUSED/LP <trackname>: load but do not start playing resource <trackname>");
 					generic_sendmsg("H PAUSE/P: pause playback");
 					generic_sendmsg("H STOP/S: stop playback (closes file)");
 					generic_sendmsg("H JUMP/J <frame>|<+offset>|<-offset>|<[+|-]seconds>s: jump to mpeg frame <frame> or change position by offset, same in seconds if number followed by \"s\"");
