@@ -79,7 +79,7 @@ void frame_init_par(mpg123_handle *fr, mpg123_pars *mp)
 	frame_fixed_reset(fr); /* Reset only the fixed data, dynamic buffers are not there yet! */
 }
 
-mpg123_pars *mpg123_new_pars(int *error)
+mpg123_pars attribute_align_arg *mpg123_new_pars(int *error)
 {
 	mpg123_pars *mp = malloc(sizeof(struct mpg123_pars_struct));
 	if(mp != NULL){ frame_default_pars(mp); if(error != NULL) *error = MPG123_OK; }
@@ -87,12 +87,12 @@ mpg123_pars *mpg123_new_pars(int *error)
 	return mp;
 }
 
-void mpg123_delete_pars(mpg123_pars* mp)
+void attribute_align_arg mpg123_delete_pars(mpg123_pars* mp)
 {
 	if(mp != NULL) free(mp);
 }
 
-int mpg123_reset_eq(mpg123_handle *mh)
+int attribute_align_arg mpg123_reset_eq(mpg123_handle *mh)
 {
 	int i;
 	mh->have_eq_settings = 0;
@@ -122,7 +122,7 @@ int frame_outbuffer(mpg123_handle *fr)
 	return 0;
 }
 
-int mpg123_replace_buffer(mpg123_handle *mh, unsigned char *data, size_t size)
+int attribute_align_arg mpg123_replace_buffer(mpg123_handle *mh, unsigned char *data, size_t size)
 {
 	if(data == NULL || size < mpg123_safe_buffer())
 	{
@@ -309,7 +309,6 @@ static void frame_fixed_reset(mpg123_handle *fr)
 	fr->index.fill = 0;
 	fr->index.step = 1;
 	fr->fsizeold = 0;
-	fr->do_recover = 0;
 	fr->firstframe = 0;
 	fr->ignoreframe = fr->firstframe-IGNORESHIFT;
 	fr->lastframe = -1;
@@ -352,7 +351,7 @@ void frame_exit(mpg123_handle *fr)
 	clear_icy(&fr->icy);
 }
 
-int mpg123_info(mpg123_handle *mh, struct mpg123_frameinfo *mi)
+int attribute_align_arg mpg123_info(mpg123_handle *mh, struct mpg123_frameinfo *mi)
 {
 	if(mh == NULL) return MPG123_ERR;
 	if(mi == NULL)
@@ -529,16 +528,6 @@ void frame_set_seek(mpg123_handle *fr, off_t sp)
 #endif
 }
 
-/* Unadjusted! */
-off_t frame_tell_seek(mpg123_handle *fr)
-{
-	off_t pos = frame_outs(fr, fr->firstframe);
-#ifdef GAPLESS
-	pos += fr->firstoff;
-#endif
-	return pos;
-}
-
 /* to vanish */
 void frame_outformat(mpg123_handle *fr, int format, int channels, long rate)
 {
@@ -604,13 +593,13 @@ int set_synth_functions(mpg123_handle *fr)
 	return 0;
 }
 
-int mpg123_volume_change(mpg123_handle *mh, double change)
+int attribute_align_arg mpg123_volume_change(mpg123_handle *mh, double change)
 {
 	if(mh == NULL) return MPG123_ERR;
 	return mpg123_volume(mh, change + (double) mh->p.outscale / MAXOUTBURST);
 }
 
-int mpg123_volume(mpg123_handle *mh, double vol)
+int attribute_align_arg mpg123_volume(mpg123_handle *mh, double vol)
 {
 	if(mh == NULL) return MPG123_ERR;
 	if(vol >= 0) mh->p.outscale = (double) MAXOUTBURST * vol;
@@ -659,18 +648,26 @@ void do_rva(mpg123_handle *fr)
 	if((peak*newscale) > MAXOUTBURST)
 	{
 		newscale = (scale_t) ((double) MAXOUTBURST/peak);
+#ifdef FLOATOUT
+		warning2("limiting scale value to %f to prevent clipping with indicated peak factor of %f", newscale, peak);
+#else
 		warning2("limiting scale value to %li to prevent clipping with indicated peak factor of %f", newscale, peak);
+#endif
 	}
 	/* first rva setting is forced with fr->lastscale < 0 */
 	if(newscale != fr->lastscale)
 	{
+#ifdef FLOATOUT
+		debug3("changing scale value from %f to %f (peak estimated to %f)", fr->lastscale != -1 ? fr->lastscale : fr->p.outscale, newscale, (double) (newscale*peak));
+#else
 		debug3("changing scale value from %li to %li (peak estimated to %li)", fr->lastscale != -1 ? fr->lastscale : fr->p.outscale, newscale, (long) (newscale*peak));
+#endif
 		fr->lastscale = newscale;
 		opt_make_decode_tables(fr); /* the actual work */
 	}
 }
 
-int mpg123_getvolume(mpg123_handle *mh, double *base, double *really, double *rva_db)
+int attribute_align_arg mpg123_getvolume(mpg123_handle *mh, double *base, double *really, double *rva_db)
 {
 	if(mh == NULL) return MPG123_ERR;
 	if(base)   *base   = (double)mh->p.outscale/MAXOUTBURST;
