@@ -1,7 +1,7 @@
 /*
 	leyer3.c: the layer 3 decoder
 
-	copyright 1995-2007 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1995-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 
@@ -13,8 +13,8 @@
 
 #include "mpg123lib_intern.h"
 #include "huffman.h"
-
 #include "getbits.h"
+#include "debug.h"
 
 /* static one-time calculated tables... or so */
 static real ispow[8207];
@@ -94,9 +94,9 @@ const struct bandInfoStruct bandInfo[9] = {
    {0,4*3,8*3,12*3,18*3,24*3,32*3,42*3,56*3,74*3,100*3,132*3,174*3,192*3} ,
    {4,4,4,6,6,8,10,14,18,26,32,42,18 } } ,
 
-/* mhipp trunk has 330 -> 332 without further explanation ... */
- { {0,6,12,18,24,30,36,44,54,66,80,96,114,136,162,194,232,278,330,394,464,540,576},
-   {6,6,6,6,6,6,8,10,12,14,16,18,22,26,32,38,46,52,64,70,76,36 } ,
+/* Twiddling 3 values here (not just 330->332!) fixed bug 1895025. */
+ { {0,6,12,18,24,30,36,44,54,66,80,96,114,136,162,194,232,278,332,394,464,540,576},
+   {6,6,6,6,6,6,8,10,12,14,16,18,22,26,32,38,46,54,62,70,76,36 } ,
    {0,4*3,8*3,12*3,18*3,26*3,36*3,48*3,62*3,80*3,104*3,136*3,180*3,192*3} ,
    {4,4,4,6,8,10,12,14,18,24,32,44,12 } } ,
 
@@ -441,6 +441,10 @@ static int III_get_side_info(mpg123_handle *fr, struct III_sideinfo *si,int ster
          r1c = getbits_fast(fr, 3);
          gr_info->region1start = bandInfo[sfreq].longIdx[r0c+1] >> 1 ;
          gr_info->region2start = bandInfo[sfreq].longIdx[r0c+1+r1c+1] >> 1;
+         if(r0c + r1c + 2 > 22)
+           gr_info->region2start = 576>>1;
+         else
+           gr_info->region2start = bandInfo[sfreq].longIdx[r0c+1+r1c+1] >> 1;
          gr_info->block_type = 0;
          gr_info->mixed_block_flag = 0;
        }
@@ -614,7 +618,10 @@ static int III_dequantize_sample(mpg123_handle *fr, real xr[SBLIMIT][SSLIMIT],in
 
   /* mhipp tree has this split up a bit... */
   int num=getbitoffset(fr);
-  long mask = (long) getbits(fr, num)<<(BITSHIFT+8-num);
+  long mask;
+  /* We must split this, because for num==0 the shift is undefined if you do it in one step. */
+  mask  = ((unsigned long) getbits(fr, num))<<BITSHIFT;
+  mask <<= 8-num;
   part2remain -= num;
 
   {
@@ -1849,14 +1856,6 @@ int do_layer3(mpg123_handle *fr)
         clip += (fr->synth)(hybridOut[1][ss], 1, fr, 1);
       }
 
-#ifdef VARMODESUPPORT
-      if (playlimit < 128) {
-        fr->buffer.fill -= playlimit >> 1;
-        playlimit = 0;
-      }
-      else
-        playlimit -= 128;
-#endif
     }
 #ifdef OPT_I486
     } else {
