@@ -7,6 +7,7 @@
 */
 
 #include "mpg123lib_intern.h"
+#include "icy2utf8.h"
 #include "getbits.h"
 #include "debug.h"
 
@@ -22,6 +23,32 @@
 
 static int initialized = 0;
 
+#define ALIGNCHECK(mh)
+#define ALIGNCHECKK
+#ifdef DEBUG
+#ifdef CCALIGN
+#undef ALIGNCHECK
+#define ALIGNCHECK(mh) \
+	double ALIGNED(16) altest[4]; \
+	if(((size_t)altest) % 16 != 0) \
+	{ \
+		error("Stack variable is not aligned! Your combination of compiler/library is dangerous!"); \
+		if(mh != NULL) mh->err = MPG123_BAD_ALIGN; \
+\
+		return MPG123_ERR; \
+	}
+#undef ALIGNCHECKK
+#define ALIGNCHECKK \
+	double ALIGNED(16) altest[4]; \
+	if(((size_t)altest) % 16 != 0) \
+	{ \
+		error("Stack variable is not aligned! Your combination of compiler/library is dangerous!"); \
+		return MPG123_BAD_ALIGN; \
+	}
+#endif
+#endif
+
+
 #ifdef GAPLESS
 /*
 	Take the buffer after a frame decode (strictly: it is the data from frame fr->num!) and cut samples out.
@@ -61,8 +88,9 @@ static void frame_buffercheck(mpg123_handle *fr)
 #endif
 
 
-int mpg123_init(void)
+int attribute_align_arg mpg123_init(void)
 {
+	ALIGNCHECKK
 	if((sizeof(short) != 2) || (sizeof(long) < 4)) return MPG123_BAD_TYPES;
 
 	init_layer2(); /* inits also shared tables with layer1 */
@@ -75,23 +103,34 @@ int mpg123_init(void)
 	return MPG123_OK;
 }
 
-void mpg123_exit(void)
+void attribute_align_arg mpg123_exit(void)
 {
 	/* nothing yet, but something later perhaps */
 	if(initialized) return;
 }
 
 /* create a new handle with specified decoder, decoder can be "", "auto" or NULL for auto-detection */
-mpg123_handle *mpg123_new(const char* decoder, int *error)
+mpg123_handle attribute_align_arg *mpg123_new(const char* decoder, int *error)
 {
 	return mpg123_parnew(NULL, decoder, error);
 }
 
 /* ...the full routine with optional initial parameters to override defaults. */
-mpg123_handle *mpg123_parnew(mpg123_pars *mp, const char* decoder, int *error)
+mpg123_handle attribute_align_arg *mpg123_parnew(mpg123_pars *mp, const char* decoder, int *error)
 {
 	mpg123_handle *fr = NULL;
 	int err = MPG123_OK;
+#ifdef DEBUG
+#ifdef CCALIGN
+	double ALIGNED(16) altest[4];
+	if(((size_t)altest) % 16 != 0)
+	{
+		error("Stack variable is not aligned! Your combination of compiler/library is dangerous!");
+		*error = MPG123_BAD_ALIGN;
+		return NULL;
+	}
+#endif
+#endif
 	if(initialized) fr = (mpg123_handle*) malloc(sizeof(mpg123_handle));
 	else err = MPG123_NOT_INITIALIZED;
 	if(fr != NULL)
@@ -130,9 +169,10 @@ mpg123_handle *mpg123_parnew(mpg123_pars *mp, const char* decoder, int *error)
 	return fr;
 }
 
-int mpg123_decoder(mpg123_handle *mh, const char* decoder)
+int attribute_align_arg mpg123_decoder(mpg123_handle *mh, const char* decoder)
 {
 	enum optdec dt = dectype(decoder);
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 
 	if(dt == nodec)
@@ -164,18 +204,20 @@ int mpg123_decoder(mpg123_handle *mh, const char* decoder)
 	return MPG123_OK;
 }
 
-int mpg123_param(mpg123_handle *mh, enum mpg123_parms key, long val, double fval)
+int attribute_align_arg mpg123_param(mpg123_handle *mh, enum mpg123_parms key, long val, double fval)
 {
 	int r;
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	r = mpg123_par(&mh->p, key, val, fval);
 	if(r != MPG123_OK){ mh->err = r; r = MPG123_ERR; }
 	return r;
 }
 
-int mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long val, double fval)
+int attribute_align_arg mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long val, double fval)
 {
 	int ret = MPG123_OK;
+	ALIGNCHECKK
 	if(mp == NULL) return MPG123_BAD_PARS;
 	switch(key)
 	{
@@ -185,9 +227,11 @@ int mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long val, double fval)
 		case MPG123_FLAGS:
 #ifndef GAPLESS
 			if(val & MPG123_GAPLESS) ret = MPG123_NO_GAPLESS;
-			else
 #endif
-			mp->flags = val;
+#ifdef FLOATOUT
+			if(val & MPG123_FORCE_8BIT) ret = MPG123_NO_8BIT;
+#endif
+			if(ret == MPG123_OK) mp->flags = val;
 			debug1("set flags to 0x%lx", (unsigned long) mp->flags);
 		break;
 		case MPG123_ADD_FLAGS:
@@ -247,18 +291,20 @@ int mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long val, double fval)
 	return ret;
 }
 
-int mpg123_getparam(mpg123_handle *mh, enum mpg123_parms key, long *val, double *fval)
+int attribute_align_arg mpg123_getparam(mpg123_handle *mh, enum mpg123_parms key, long *val, double *fval)
 {
 	int r;
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	r = mpg123_getpar(&mh->p, key, val, fval);
 	if(r != MPG123_OK){ mh->err = r; r = MPG123_ERR; }
 	return r;
 }
 
-int mpg123_getpar(mpg123_pars *mp, enum mpg123_parms key, long *val, double *fval)
+int attribute_align_arg mpg123_getpar(mpg123_pars *mp, enum mpg123_parms key, long *val, double *fval)
 {
 	int ret = 0;
+	ALIGNCHECKK
 	if(mp == NULL) return MPG123_BAD_PARS;
 	switch(key)
 	{
@@ -303,8 +349,9 @@ int mpg123_getpar(mpg123_pars *mp, enum mpg123_parms key, long *val, double *fva
 	return ret;
 }
 
-int mpg123_eq(mpg123_handle *mh, enum mpg123_channels channel, int band, double val)
+int attribute_align_arg mpg123_eq(mpg123_handle *mh, enum mpg123_channels channel, int band, double val)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(band < 0 || band > 31){ mh->err = MPG123_BAD_BAND; return MPG123_ERR; }
 	switch(channel)
@@ -324,8 +371,9 @@ int mpg123_eq(mpg123_handle *mh, enum mpg123_channels channel, int band, double
 
 
 /* plain file access, no http! */
-int mpg123_open(mpg123_handle *mh, char *path)
+int attribute_align_arg mpg123_open(mpg123_handle *mh, char *path)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 
 	mpg123_close(mh);
@@ -333,8 +381,9 @@ int mpg123_open(mpg123_handle *mh, char *path)
 	return open_stream(mh, path, -1);
 }
 
-int mpg123_open_fd(mpg123_handle *mh, int fd)
+int attribute_align_arg mpg123_open_fd(mpg123_handle *mh, int fd)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 
 	mpg123_close(mh);
@@ -342,8 +391,9 @@ int mpg123_open_fd(mpg123_handle *mh, int fd)
 	return open_stream(mh, NULL, fd);
 }
 
-int mpg123_open_feed(mpg123_handle *mh)
+int attribute_align_arg mpg123_open_feed(mpg123_handle *mh)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 
 	mpg123_close(mh);
@@ -351,10 +401,11 @@ int mpg123_open_feed(mpg123_handle *mh)
 	return open_feed(mh);
 }
 
-int mpg123_replace_reader( mpg123_handle *mh,
+int attribute_align_arg mpg123_replace_reader( mpg123_handle *mh,
                            ssize_t (*r_read) (int, void *, size_t),
                            off_t   (*r_lseek)(int, off_t, int) )
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	mh->rdat.r_read = r_read;
 	mh->rdat.r_lseek = r_lseek;
@@ -364,7 +415,9 @@ int mpg123_replace_reader( mpg123_handle *mh,
 
 int decode_update(mpg123_handle *mh)
 {
-	long native_rate = frame_freq(mh);
+	long native_rate;
+	ALIGNCHECK(mh);
+	native_rate = frame_freq(mh);
 	debug2("updating decoder structure with native rate %li and af.rate %li", native_rate, mh->af.rate);
 	if(mh->af.rate == native_rate) mh->down_sample = 0;
 	else if(mh->af.rate == native_rate>>1) mh->down_sample = 1;
@@ -411,12 +464,12 @@ int decode_update(mpg123_handle *mh)
 	return 0;
 }
 
-size_t mpg123_safe_buffer()
+size_t attribute_align_arg mpg123_safe_buffer()
 {
 	return sizeof(sample_t)*2*1152*NTOM_MAX;
 }
 
-size_t mpg123_outblock(mpg123_handle *mh)
+size_t attribute_align_arg mpg123_outblock(mpg123_handle *mh)
 {
 	if(mh != NULL) return mh->outblock;
 	else return mpg123_safe_buffer();
@@ -503,8 +556,9 @@ static int get_next_frame(mpg123_handle *mh)
 
 	num will be updated to the last decoded frame number (may possibly _not_ increase, p.ex. when format changed).
 */
-int mpg123_decode_frame(mpg123_handle *mh, off_t *num, unsigned char **audio, size_t *bytes)
+int attribute_align_arg mpg123_decode_frame(mpg123_handle *mh, off_t *num, unsigned char **audio, size_t *bytes)
 {
+	ALIGNCHECK(mh);
 	if(bytes != NULL) *bytes = 0;
 	if(mh == NULL) return MPG123_ERR;
 	if(mh->buffer.size < mh->outblock) return MPG123_NO_SPACE;
@@ -543,11 +597,30 @@ int mpg123_decode_frame(mpg123_handle *mh, off_t *num, unsigned char **audio, si
 	return MPG123_ERR;
 }
 
-int mpg123_read(mpg123_handle *mh, unsigned char *out, size_t size, size_t *done)
+int attribute_align_arg mpg123_read(mpg123_handle *mh, unsigned char *out, size_t size, size_t *done)
 {
 	return mpg123_decode(mh, NULL, 0, out, size, done);
 }
 
+int attribute_align_arg mpg123_feed(mpg123_handle *mh, const unsigned char *in, size_t size)
+{
+	if(mh == NULL) return MPG123_ERR;
+	if(size > 0)
+	{
+		if(in != NULL)
+		{
+			if(feed_more(mh, in, size) != 0) return MPG123_ERR;
+			else return MPG123_OK;
+		}
+		else
+		{
+			mh->err = MPG123_NULL_BUFFER;
+			return MPG123_ERR;
+		}
+	}
+	return MPG123_OK;
+}
+
 /*
 	The old picture:
 	while(1) {
@@ -562,19 +635,23 @@ int mpg123_read(mpg123_handle *mh, unsigned char *out, size_t size, size_t *done
 	}
 */
 
-int mpg123_decode(mpg123_handle *mh,unsigned char *inmemory, size_t inmemsize, unsigned char *outmemory, size_t outmemsize, size_t *done)
+int attribute_align_arg mpg123_decode(mpg123_handle *mh, const unsigned char *inmemory, size_t inmemsize, unsigned char *outmemory, size_t outmemsize, size_t *done)
 {
 	int ret = MPG123_OK;
 	size_t mdone = 0;
+	ALIGNCHECK(mh);
 	if(done != NULL) *done = 0;
 	if(mh == NULL) return MPG123_ERR;
-	if(inmemsize > 0)
-	if(feed_more(mh, inmemory, inmemsize) == -1){ ret = MPG123_ERR; goto decodeend; }
+	if(inmemsize > 0 && mpg123_feed(mh, inmemory, inmemsize) != MPG123_OK)
+	{
+		ret = MPG123_ERR;
+		goto decodeend;
+	}
 	if(outmemory == NULL) outmemsize = 0; /* Not just give error, give chance to get a status message. */
 
 	while(ret == MPG123_OK)
 	{
-		debug3("decode loop, fill %i (%li vs. %li)", (int)mh->buffer.fill, (long)mh->num, (long)mh->firstframe);
+		debug4("decode loop, fill %i (%li vs. %li); to_decode: %i", (int)mh->buffer.fill, (long)mh->num, (long)mh->firstframe, mh->to_decode);
 		/* Decode a frame that has been read before.
 		   This only happens when buffer is empty! */
 		if(mh->to_decode)
@@ -601,7 +678,7 @@ int mpg123_decode(mpg123_handle *mh,unsigned char *inmemory, size_t inmemsize, u
 		{
 			/* get what is needed - or just what is there */
 			int a = mh->buffer.fill > (outmemsize - mdone) ? outmemsize - mdone : mh->buffer.fill;
-			debug4("buffer fill: %i; copying %i (%i - %li)", (int)mh->buffer.fill, a, (int)outmemsize, (long)*done);
+			debug4("buffer fill: %i; copying %i (%i - %li)", (int)mh->buffer.fill, a, (int)outmemsize, (long)mdone);
 			memcpy(outmemory, mh->buffer.p, a);
 			/* less data in frame buffer, less needed, output pointer increase, more data given... */
 			mh->buffer.fill -= a;
@@ -621,9 +698,10 @@ decodeend:
 	return ret;
 }
 
-long mpg123_clip(mpg123_handle *mh)
+long attribute_align_arg mpg123_clip(mpg123_handle *mh)
 {
 	long ret = 0;
+	ALIGNCHECK(mh);
 	if(mh != NULL)
 	{
 		ret = mh->clip;
@@ -645,8 +723,9 @@ static int init_track(mpg123_handle *mh)
 	return 0;
 }
 
-int mpg123_getformat(mpg123_handle *mh, long *rate, int *channels, int *encoding)
+int attribute_align_arg mpg123_getformat(mpg123_handle *mh, long *rate, int *channels, int *encoding)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(init_track(mh) == MPG123_ERR) return MPG123_ERR;
 
@@ -657,9 +736,10 @@ int mpg123_getformat(mpg123_handle *mh, long *rate, int *channels, int *encoding
 	return MPG123_OK;
 }
 
-off_t mpg123_timeframe(mpg123_handle *mh, double seconds)
+off_t attribute_align_arg mpg123_timeframe(mpg123_handle *mh, double seconds)
 {
 	off_t b;
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	b = init_track(mh);
 	if(b<0) return b;
@@ -674,19 +754,35 @@ off_t mpg123_timeframe(mpg123_handle *mh, double seconds)
 	Then, there is firstframe...when we didn't reach it yet, then the next data will come from there.
 	mh->num starts with -1
 */
-off_t mpg123_tell(mpg123_handle *mh)
+off_t attribute_align_arg mpg123_tell(mpg123_handle *mh)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(track_need_init(mh)) return 0;
 	/* Now we have all the info at hand. */
-	debug4("tell: %li/%i first %li buffer %lu", (long)mh->num, mh->to_decode, (long)mh->firstframe, (unsigned long)mh->buffer.fill);
-	if((mh->num < mh->firstframe) || (mh->num == mh->firstframe && mh->to_decode)) return SAMPLE_ADJUST(frame_tell_seek(mh));
-	else if(mh->to_decode) return SAMPLE_ADJUST(frame_outs(mh, mh->num) - mh->buffer.fill);
-	else return SAMPLE_ADJUST(frame_outs(mh, mh->num+1) - mh->buffer.fill);
+	debug5("tell: %li/%i first %li buffer %lu; frame_outs=%li", (long)mh->num, mh->to_decode, (long)mh->firstframe, (unsigned long)mh->buffer.fill, (long)frame_outs(mh, mh->num));
+
+	if((mh->num < mh->firstframe) || (mh->num == mh->firstframe && mh->to_decode))
+	{ /* We are at the beginning, expect output from firstframe on. */
+		off_t pos = frame_outs(mh, mh->firstframe);
+#ifdef GAPLESS
+		pos += mh->firstoff;
+#endif
+		return SAMPLE_ADJUST(pos);
+	}
+	else if(mh->to_decode)
+	{ /* We start fresh with this frame. Buffer should be empty, but we make sure to count it in.  */
+		return SAMPLE_ADJUST(frame_outs(mh, mh->num) - bytes_to_samples(mh, mh->buffer.fill));
+	}
+	else
+	{ /* We serve what we have in buffer and then the beginning of next frame... */
+		return SAMPLE_ADJUST(frame_outs(mh, mh->num+1) - bytes_to_samples(mh, mh->buffer.fill));
+	}
 }
 
-off_t mpg123_tellframe(mpg123_handle *mh)
+off_t attribute_align_arg mpg123_tellframe(mpg123_handle *mh)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(mh->num < mh->firstframe) return mh->firstframe;
 	if(mh->to_decode) return mh->num;
@@ -714,10 +810,12 @@ static int do_the_seek(mpg123_handle *mh)
 	return 0;
 }
 
-off_t mpg123_seek(mpg123_handle *mh, off_t sampleoff, int whence)
+off_t attribute_align_arg mpg123_seek(mpg123_handle *mh, off_t sampleoff, int whence)
 {
 	int b;
-	off_t pos = mpg123_tell(mh); /* adjusted samples */
+	off_t pos;
+	ALIGNCHECK(mh);
+	pos = mpg123_tell(mh); /* adjusted samples */
 debug1("pos=%li", (long)pos);
 	if(pos < 0) return pos; /* mh == NULL is covered in mpg123_tell() */
 	if((b=init_track(mh)) < 0) return b;
@@ -755,10 +853,12 @@ debug1("pos=%li", (long)pos);
 	The caller doesn't know where a specific frame starts and mpg123 also only knows the general region after it scanned the file.
 	Well, it is tricky...
 */
-off_t mpg123_feedseek(mpg123_handle *mh, off_t sampleoff, int whence, off_t *input_offset)
+off_t attribute_align_arg mpg123_feedseek(mpg123_handle *mh, off_t sampleoff, int whence, off_t *input_offset)
 {
 	int b;
-	off_t pos = mpg123_tell(mh); /* adjusted samples */
+	off_t pos;
+	ALIGNCHECK(mh);
+	pos = mpg123_tell(mh); /* adjusted samples */
 	debug3("seek from %li to %li (whence=%i)", (long)pos, (long)sampleoff, whence);
 	if(pos < 0) return pos; /* mh == NULL is covered in mpg123_tell() */
 	if((b=init_track(mh)) < 0) return b; /* May need more to do anything at all. */
@@ -800,10 +900,11 @@ feedseekend:
 	return mpg123_tell(mh);
 }
 
-off_t mpg123_seek_frame(mpg123_handle *mh, off_t offset, int whence)
+off_t attribute_align_arg mpg123_seek_frame(mpg123_handle *mh, off_t offset, int whence)
 {
 	int b;
 	off_t pos = 0;
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if((b=init_track(mh)) < 0) return b;
 
@@ -836,21 +937,33 @@ off_t mpg123_seek_frame(mpg123_handle *mh, off_t offset, int whence)
 	return mpg123_tellframe(mh);
 }
 
-off_t mpg123_length(mpg123_handle *mh)
+int attribute_align_arg mpg123_set_filesize(mpg123_handle *mh, off_t size)
+{
+	ALIGNCHECK(mh);
+	if(mh == NULL) return MPG123_ERR;
+
+	mh->rdat.filelen = size;
+	return MPG123_OK;
+}
+
+off_t attribute_align_arg mpg123_length(mpg123_handle *mh)
 {
 	int b;
 	off_t length;
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	b = init_track(mh);
 	if(b<0) return b;
 	if(mh->track_samples > -1) length = mh->track_samples;
 	else if(mh->track_frames > 0) length = mh->track_frames*spf(mh);
-	else
+	else if(mh->rdat.filelen >= 0) /* Let the case of 0 length just fall through. */
 	{
 		/* A bad estimate. Ignoring tags 'n stuff. */
 		double bpf = mh->mean_framesize ? mh->mean_framesize : compute_bpf(mh);
 		length = (off_t)((double)(mh->rdat.filelen)/bpf*spf(mh));
 	}
+	else return MPG123_ERR; /* No length info there! */
+
 	length = frame_ins2outs(mh, length);
 #ifdef GAPLESS
 	if(mh->end_os > 0 && length > mh->end_os) length = mh->end_os;
@@ -859,11 +972,12 @@ off_t mpg123_length(mpg123_handle *mh)
 	return length;
 }
 
-int mpg123_scan(mpg123_handle *mh)
+int attribute_align_arg mpg123_scan(mpg123_handle *mh)
 {
 	int b;
 	off_t backframe;
 	int to_decode, to_ignore;
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(!(mh->rdat.flags & READER_SEEKABLE)){ mh->err = MPG123_NO_SEEK; return MPG123_ERR; }
 	/* Scan through the _whole_ file, since the current position is no count but computed assuming constant samples per frame. */
@@ -894,14 +1008,15 @@ int mpg123_scan(mpg123_handle *mh)
 	return MPG123_OK;
 }
 
-int mpg123_meta_check(mpg123_handle *mh)
+int attribute_align_arg mpg123_meta_check(mpg123_handle *mh)
 {
 	if(mh != NULL) return mh->metaflags;
 	else return 0;
 }
 
-int mpg123_id3(mpg123_handle *mh, mpg123_id3v1 **v1, mpg123_id3v2 **v2)
+int attribute_align_arg mpg123_id3(mpg123_handle *mh, mpg123_id3v1 **v1, mpg123_id3v2 **v2)
 {
+	ALIGNCHECK(mh);
 	if(v1 != NULL) *v1 = NULL;
 	if(v2 != NULL) *v2 = NULL;
 	if(mh == NULL) return MPG123_ERR;
@@ -917,8 +1032,9 @@ int mpg123_id3(mpg123_handle *mh, mpg123_id3v1 **v1, mpg123_id3v2 **v2)
 	return MPG123_OK;
 }
 
-int mpg123_icy(mpg123_handle *mh, char **icy_meta)
+int attribute_align_arg mpg123_icy(mpg123_handle *mh, char **icy_meta)
 {
+	ALIGNCHECK(mh);
 	*icy_meta = NULL;
 	if(mh == NULL) return MPG123_ERR;
 
@@ -931,8 +1047,15 @@ int mpg123_icy(mpg123_handle *mh, char **icy_meta)
 	return MPG123_OK;
 }
 
-int mpg123_index(mpg123_handle *mh, off_t **offsets, off_t *step, size_t *fill)
+char* attribute_align_arg mpg123_icy2utf8(const char* icy_text)
 {
+	return icy2utf8(icy_text);
+}
+
+
+int attribute_align_arg mpg123_index(mpg123_handle *mh, off_t **offsets, off_t *step, size_t *fill)
+{
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(offsets == NULL || step == NULL || fill == NULL) return MPG123_BAD_INDEX_PAR;
 
@@ -943,15 +1066,16 @@ int mpg123_index(mpg123_handle *mh, off_t **offsets, off_t *step, size_t *fill)
 	return MPG123_OK;
 }
 
-int mpg123_close(mpg123_handle *mh)
+int attribute_align_arg mpg123_close(mpg123_handle *mh)
 {
+	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
 	if(mh->rd != NULL && mh->rd->close != NULL) mh->rd->close(mh);
 	mh->rd = NULL;
 	return MPG123_OK;
 }
 
-void mpg123_delete(mpg123_handle *mh)
+void attribute_align_arg mpg123_delete(mpg123_handle *mh)
 {
 	if(mh != NULL)
 	{
@@ -991,10 +1115,13 @@ static const char *mpg123_error[] =
 	"Bad parameter handle. (code 25)",
 	"Invalid parameter addresses for index retrieval. (code 26)",
 	"Lost track in the bytestream and did not attempt resync. (code 27)",
-	"Failed to find valid MPEG data within limit on resync. (code 28)"
+	"Failed to find valid MPEG data within limit on resync. (code 28)",
+	"No 8bit encoding possible. (code 29)",
+	"Stack alignment is not good. (code 30)",
+	"You gave me a NULL buffer? (code 31)"
 };
 
-const char* mpg123_plain_strerror(int errcode)
+const char* attribute_align_arg mpg123_plain_strerror(int errcode)
 {
 	if(errcode >= 0 && errcode < sizeof(mpg123_error)/sizeof(char*))
 	return mpg123_error[errcode];
@@ -1013,13 +1140,13 @@ const char* mpg123_plain_strerror(int errcode)
 	}
 }
 
-int mpg123_errcode(mpg123_handle *mh)
+int attribute_align_arg mpg123_errcode(mpg123_handle *mh)
 {
 	if(mh != NULL) return mh->err;
 	return MPG123_BAD_HANDLE;
 }
 
-const char* mpg123_strerror(mpg123_handle *mh)
+const char* attribute_align_arg mpg123_strerror(mpg123_handle *mh)
 {
 	return mpg123_plain_strerror(mpg123_errcode(mh));
 }
