@@ -15,10 +15,22 @@
 struct buffy
 {
 	unsigned char *data;
-	off_t size;
+	ssize_t size;
 	struct buffy *next;
 };
 
+struct bufferchain
+{
+	struct buffy* first; /* The beginning of the chain. */
+	struct buffy* last;  /* The end...    of the chain. */
+	ssize_t size;        /* Aggregated size of all buffies. */
+	/* These positions are relative to buffer chain beginning. */
+	ssize_t pos;         /* Position in whole chain. */
+	ssize_t firstpos;    /* The point of return on non-forget() */
+	/* The "real" filepos is fileoff + pos. */
+	off_t fileoff;       /* Beginning of chain is at this file offset. */
+};
+
 struct reader_data
 {
 	off_t filelen; /* total file length or total buffer size */
@@ -35,9 +47,9 @@ struct reader_data
 	off_t   (*r_lseek)(int fd, off_t offset, int whence);
 	ssize_t (*read) (int fd, void *buf, size_t count);
 	off_t   (*lseek)(int fd, off_t offset, int whence);
-	/* variables specific to feed reader */
-	off_t firstpos; /* the point of return on non-forget() */
-	struct buffy *buf;  /* first in buffer chain */
+	/* Buffered readers want that abstracted, set internally. */
+	ssize_t (*fullread)(mpg123_handle *, unsigned char *, ssize_t);
+	struct bufferchain buffer; /* Not dynamically allocated, these few struct bytes aren't worth the trouble. */
 };
 
 /* start to use off_t to properly do LFS in future ... used to be long */
@@ -63,7 +75,7 @@ int open_stream(mpg123_handle *, char *path, int fd);
 /* feed based operation has some specials */
 int open_feed(mpg123_handle *);
 /* externally called function, returns 0 on success, -1 on error */
-int  feed_more(mpg123_handle *fr, unsigned char *in, long count);
+int  feed_more(mpg123_handle *fr, const unsigned char *in, long count);
 void feed_forget(mpg123_handle *fr);  /* forget the data that has been read (free some buffers) */
 off_t feed_set_pos(mpg123_handle *fr, off_t pos); /* Set position (inside available data if possible), return wanted byte offset of next feed. */
 
@@ -73,18 +85,20 @@ void open_bad(mpg123_handle *);
 #define READER_ID3TAG    0x2
 #define READER_SEEKABLE  0x4
 #define READER_BUFFERED  0x8
-#define READER_MICROSEEK 0x10
 #define READER_NONBLOCK  0x20
 
 #define READER_STREAM 0
 #define READER_ICY_STREAM 1
 #define READER_FEED       2
+/* These two add a little buffering to enable small seeks for peek ahead. */
+#define READER_BUF_STREAM 3
+#define READER_BUF_ICY_STREAM 4
 
 #ifdef READ_SYSTEM
-#define READER_SYSTEM 3
-#define READERS 4
+#define READER_SYSTEM 5
+#define READERS 6
 #else
-#define READERS 3
+#define READERS 5
 #endif
 
 #define READER_ERROR MPG123_ERR
