@@ -1,7 +1,7 @@
 /*
 	mpg123: main code of the program (not of the decoder...)
 
-	copyright 1995-2006 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1995-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 */
@@ -13,9 +13,6 @@
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
@@ -43,11 +40,13 @@
 #include "term.h"
 #include "playlist.h"
 #include "httpget.h"
-#include "id3print.h"
+#include "metaprint.h"
 #include "getbits.h"
 #include "httpget.h"
 #include "getcpuflags.h"
 
+#include "debug.h"
+
 static void usage(int err);
 static void want_usage(char* arg);
 static void long_usage(int err);
@@ -103,6 +102,10 @@ struct parameter param = {
 	,0 /* force_rate */
 	,1 /* ICY */
 	,1024 /* resync_limit */
+	,0 /* smooth */
+	,0.0 /* pitch */
+	,0 /* ignore_mime */
+	,NULL /* proxyurl */
 };
 
 int utf8env = 0;
@@ -169,7 +172,7 @@ void safe_exit(int code)
 	if(mh != NULL) mpg123_delete(mh);
 
 	mpg123_exit();
-	httpdata_reset(&htd);
+	httpdata_free(&htd);
 	/* It's ugly... but let's just fix this still-reachable memory chunk of static char*. */
 	split_dir_file("", &dummy, &dammy);
 	exit(code);
@@ -335,7 +338,9 @@ topt opts[] = {
 	{'c', "check",       GLO_INT,  0, &param.checkrange, TRUE},
 	{'v', "verbose",     0,        set_verbose, 0,           0},
 	{'q', "quiet",       0,        set_quiet,   0,           0},
-	{'y', "resync",      GLO_INT,  set_frameflag, &frameflag, MPG123_NO_RESYNC},
+	{'y', "no-resync",      GLO_INT,  set_frameflag, &frameflag, MPG123_NO_RESYNC},
+	/* compatibility, no-resync is to be used nowadays */
+	{0, "resync",      GLO_INT,  set_frameflag, &frameflag, MPG123_NO_RESYNC},
 	{'0', "single0",     GLO_INT,  set_frameflag, &frameflag, MPG123_MONO_LEFT},
 	{0,   "left",        GLO_INT,  set_frameflag, &frameflag, MPG123_MONO_LEFT},
 	{'1', "single1",     GLO_INT,  set_frameflag, &frameflag, MPG123_MONO_RIGHT},
@@ -363,12 +368,15 @@ topt opts[] = {
 	#ifdef HAVE_TERMIOS
 	{'C', "control",     GLO_INT,  0, &param.term_ctrl, TRUE},
 	#endif
+#ifndef NOXFERMEM
 	{'b', "buffer",      GLO_ARG | GLO_LONG, 0, &param.usebuffer,  0},
+	{0,  "smooth",      GLO_INT,  0, &param.smooth, 1},
+#endif
 	{'R', "remote",      GLO_INT,  0, &param.remote, TRUE},
 	{0,   "remote-err",  GLO_INT,  0, &param.remote_err, TRUE},
 	{'d', "doublespeed", GLO_ARG | GLO_LONG, 0, &param.doublespeed, 0},
 	{'h', "halfspeed",   GLO_ARG | GLO_LONG, 0, &param.halfspeed, 0},
-	{'p', "proxy",       GLO_ARG | GLO_CHAR, 0, &htd.proxyurl,   0},
+	{'p', "proxy",       GLO_ARG | GLO_CHAR, 0, &param.proxyurl,   0},
 	{'@', "list",        GLO_ARG | GLO_CHAR, 0, &param.listname,   0},
 	/* 'z' comes from the the german word 'zufall' (eng: random) */
 	{'z', "shuffle",     GLO_INT,  0, &param.shuffle, 1},
@@ -387,7 +395,7 @@ topt opts[] = {
 	{0, "cpu", GLO_ARG | GLO_CHAR, 0, &param.cpu,  0},
 	{0, "test-cpu",  GLO_INT,  0, &param.test_cpu, TRUE},
 	{0, "list-cpu", GLO_INT,  0, &param.list_cpu , 1},
-	#if !defined(WIN32) && !defined(GENERIC)
+	#ifdef NETWORK
 	{'u', "auth",        GLO_ARG | GLO_CHAR, 0, &httpauth,   0},
 	#endif
 	#ifdef HAVE_SCHED_SETSCHEDULER
@@ -422,6 +430,8 @@ topt opts[] = {
 	{'i', "index", GLO_INT, 0, &param.index, 1},
 	{'D', "delay", GLO_ARG | GLO_INT, 0, &param.delay, 0},
 	{0, "resync-limit", GLO_ARG | GLO_LONG, 0, &param.resync_limit, 0},
+	{0, "pitch", GLO_ARG|GLO_DOUBLE, 0, &param.pitch, 0},
+	{0, "ignore-mime", GLO_INT,  0, &param.ignore_mime, 1 },
 	{0, 0, 0, 0, 0, 0}
 };
 
@@ -447,7 +457,7 @@ static void reset_audio(long rate, int channels, int format)
 		buffermem->freeindex = 0;
 		if (intflag)
 			return;
-		buffermem->rate     = rate; 
+		buffermem->rate     = pitch_rate(rate); 
 		buffermem->channels = channels; 
 		buffermem->format   = format;
 		buffer_reset();
@@ -460,7 +470,7 @@ static void reset_audio(long rate, int channels, int format)
 			error("Audio handle should not be NULL here!");
 			safe_exit(98);
 		}
-		ao->rate     = rate; 
+		ao->rate     = pitch_rate(rate); 
 		ao->channels = channels; 
 		ao->format   = format;
 		if(reset_output(ao) < 0)
@@ -477,21 +487,26 @@ static void reset_audio(long rate, int channels, int format)
 int open_track(char *fname)
 {
 	filept=-1;
+	httpdata_reset(&htd);
 	if(MPG123_OK != mpg123_param(mh, MPG123_ICY_INTERVAL, 0, 0))
 	error1("Cannot (re)set ICY interval: %s", mpg123_strerror(mh));
 	if(!strcmp(fname, "-")) filept = STDIN_FILENO;
 	else if (!strncmp(fname, "http://", 7)) /* http stream */
 	{
-		httpdata_reset(&htd);
 		filept = http_open(fname, &htd);
 		/* now check if we got sth. and if we got sth. good */
 		if(    (filept >= 0) && (htd.content_type.p != NULL)
-			  && strcmp(htd.content_type.p, "audio/mpeg") && strcmp(htd.content_type.p, "audio/x-mpeg") )
+			  && !param.ignore_mime && strcmp(htd.content_type.p, "audio/mpeg") && strcmp(htd.content_type.p, "audio/x-mpeg") )
 		{
 			error1("Unknown mpeg MIME type %s - is it perhaps a playlist (use -@)?", htd.content_type.p == NULL ? "<nil>" : htd.content_type.p);
 			error("If you know the stream is mpeg1/2 audio, then please report this as "PACKAGE_NAME" bug");
 			return 0;
 		}
+		if(filept < 0)
+		{
+			error1("Access to http resource %s failed.", fname);
+			return 0;
+		}
 		if(MPG123_OK != mpg123_param(mh, MPG123_ICY_INTERVAL, htd.icy_interval, 0))
 		error1("Cannot set ICY interval: %s", mpg123_strerror(mh));
 		if(param.verbose > 1) fprintf(stderr, "Info: ICY interval %li\n", (long)htd.icy_interval);
@@ -537,7 +552,7 @@ int play_frame(void)
 	if(bytes)
 	{
 		if(param.frame_number > -1) --frames_left;
-		if(fresh && framenum == param.start_frame)
+		if(fresh && framenum >= param.start_frame)
 		{
 			fresh = FALSE;
 			if(!param.quiet)
@@ -584,6 +599,72 @@ int play_frame(void)
 	return 1;
 }
 
+void buffer_drain(void)
+{
+#ifndef NOXFERMEM
+	int s;
+	while ((s = xfermem_get_usedspace(buffermem)))
+	{
+		struct timeval wait170 = {0, 170000};
+		if(intflag) break;
+		buffer_ignore_lowmem();
+		if(param.verbose) print_stat(mh,0,s);
+	#ifdef HAVE_TERMIOS
+		if(param.term_ctrl) term_control(mh, ao);
+	#endif
+		select(0, NULL, NULL, NULL, &wait170);
+	}
+#endif
+}
+
+/* Return TRUE if we should continue (second interrupt happens quickly), skipping tracks, or FALSE if we should die. */
+#if !defined(WIN32) && !defined(GENERIC)
+int skip_or_die(struct timeval *start_time)
+{
+/* 
+ * When HAVE_TERMIOS is defined, there is 'q' to terminate a list of songs, so
+ * no pressing need to keep up this first second SIGINT hack that was too
+ * often mistaken as a bug. [dk]
+ * ThOr: Yep, I deactivated the Ctrl+C hack for active control modes.
+ *       Though, some sort of hack remains, still using intflag for track skip.
+ */
+#ifdef HAVE_TERMIOS
+	if(!param.term_ctrl)
+#endif
+	{
+		struct timeval now;
+		unsigned long secdiff;
+		gettimeofday (&now, NULL);
+		secdiff = (now.tv_sec - start_time->tv_sec) * 1000;
+		if(now.tv_usec >= start_time->tv_usec)
+		secdiff += (now.tv_usec - start_time->tv_usec) / 1000;
+		else
+		secdiff -= (start_time->tv_usec - now.tv_usec) / 1000;
+		if (secdiff < 1000)
+		{
+			debug("got the second interrupt: out of here!");
+			return FALSE;
+		}
+		else
+		{
+			debug("It's a track advancement message.");
+			++skip_tracks;
+		}
+	}
+#ifdef HAVE_TERMIOS
+	else if(skip_tracks == 0)
+	{
+		debug("breaking up");
+		return FALSE;
+	}
+#endif
+	return TRUE; /* Track advancement... no instant kill on generic/windows... */
+}
+#else
+/* On generic systems and win32, there is no decision here... just TRUE. */
+#define skip_or_die(a) TRUE
+#endif
+
 int main(int argc, char *argv[])
 {
 	int result;
@@ -592,8 +673,7 @@ int main(int argc, char *argv[])
 	int libpar = 0;
 	mpg123_pars *mp;
 #if !defined(WIN32) && !defined(GENERIC)
-	struct timeval start_time, now;
-	unsigned long secdiff;
+	struct timeval start_time;
 #endif
 	{
 		char *lang = getenv("LANG");
@@ -708,6 +788,8 @@ int main(int argc, char *argv[])
 
 	/* Set the frame parameters from command line options */
 	if(param.quiet) param.flags |= MPG123_QUIET;
+
+	param.flags |= MPG123_SEEKBUFFER; /* For HTTP streams. */
 #ifdef OPT_3DNOW
 	if(dnow != 0) param.cpu = (dnow == SET_3DNOW) ? "3dnow" : "i586";
 #endif
@@ -820,7 +902,11 @@ int main(int argc, char *argv[])
 		ret = control_generic(mh);
 		safe_exit(ret);
 	}
-
+#ifdef HAVE_TERMIOS
+		debug1("param.term_ctrl: %i", param.term_ctrl);
+		if(param.term_ctrl)
+			term_init();
+#endif
 	while ((fname = get_next_file()))
 	{
 		char *dirname, *filename;
@@ -833,6 +919,7 @@ int main(int argc, char *argv[])
 		}
 		if(param.delay > 0)
 		{
+			/* One should enable terminal control during that sleeping phase! */
 			if(param.verbose > 2) fprintf(stderr, "Note: pausing %i seconds before next track.\n", param.delay);
 			output_pause(ao);
 #ifdef WIN32
@@ -845,7 +932,17 @@ int main(int argc, char *argv[])
 		frames_left = param.frame_number;
 		debug1("Going to play %s", strcmp(fname, "-") ? fname : "standard input");
 
-		if(!open_track(fname)) continue;
+		if(intflag || !open_track(fname))
+		{
+#ifdef HAVE_TERMIOS
+			/* We need the opportunity to cancel in case of --loop -1 . */
+			if(param.term_ctrl) term_control(mh, ao);
+#endif
+			/* No wait for a second interrupt before we started playing. */
+			if(intflag) break;
+
+			continue;
+		}
 
 		if(!param.quiet) fprintf(stderr, "\n");
 		if(param.index)
@@ -857,15 +954,25 @@ int main(int argc, char *argv[])
 		if(framenum < 0)
 		{
 			error1("Initial seek failed: %s", mpg123_strerror(mh));
+			if(mpg123_errcode(mh) == MPG123_BAD_OUTFORMAT)
+			{
+				fprintf(stderr, "%s", "So, you have trouble getting an output format... this is the matrix of currently possible formats:\n");
+				print_capabilities(ao, mh);
+				fprintf(stderr, "%s", "Somehow the input data and your choices don't allow one of these.\nTried enabling resampling with a forced rate (only 2to1 and 4to1 downsampling is tried automatically)?\nTried allowing mono/stereo?\n");
+			}
 			mpg123_close(mh);
 			continue;
 		}
 
-		if (!param.quiet) {
+		if (!param.quiet)
+		{
 			if (split_dir_file(fname ? fname : "standard input",
 				&dirname, &filename))
 				fprintf(stderr, "Directory: %s\n", dirname);
+
 			fprintf(stderr, "Playing MPEG stream %lu of %lu: %s ...\n", (unsigned long)pl.pos, (unsigned long)pl.fill, filename);
+			if(htd.icy_name.fill) fprintf(stderr, "ICY-NAME: %s\n", htd.icy_name.p);
+			if(htd.icy_url.fill)  fprintf(stderr, "ICY-URL: %s\n",  htd.icy_url.p);
 
 #if !defined(GENERIC)
 {
@@ -880,7 +987,7 @@ int main(int argc, char *argv[])
 #endif
 
 		}
-
+/* Rethink that SIGINT logic... */
 #if !defined(WIN32) && !defined(GENERIC)
 #ifdef HAVE_TERMIOS
 		if(!param.term_ctrl)
@@ -888,11 +995,6 @@ int main(int argc, char *argv[])
 			gettimeofday (&start_time, NULL);
 #endif
 
-#ifdef HAVE_TERMIOS
-		debug1("param.term_ctrl: %i", param.term_ctrl);
-		if(param.term_ctrl)
-			term_init();
-#endif
 		while(!intflag)
 		{
 			int meta;
@@ -907,10 +1009,8 @@ int main(int argc, char *argv[])
 				meta = mpg123_meta_check(mh);
 				if(meta & (MPG123_NEW_ID3|MPG123_NEW_ICY))
 				{
-					char *icy;
 					if(meta & MPG123_NEW_ID3) print_id3_tag(mh, param.long_id3, stderr);
-					if(meta & MPG123_NEW_ICY && MPG123_OK == mpg123_icy(mh, &icy))
-					fprintf(stderr, "\nICY-META: %s\n", icy);
+					if(meta & MPG123_NEW_ICY) print_icy(mh, stderr);
 				}
 			}
 			if(!fresh && param.verbose)
@@ -924,30 +1024,12 @@ int main(int argc, char *argv[])
 			}
 #ifdef HAVE_TERMIOS
 			if(!param.term_ctrl) continue;
-			else term_control(mh);
+			else term_control(mh, ao);
 #endif
 		}
 
-#ifndef NOXFERMEM
-	if(param.usebuffer) {
-		int s;
-		while ((s = xfermem_get_usedspace(buffermem)))
-		{
-			struct timeval wait170 = {0, 170000};
-			if(intflag) break;
-			buffer_ignore_lowmem();
-			if(param.verbose) print_stat(mh,0,s);
-#ifdef HAVE_TERMIOS
-			if(param.term_ctrl) term_control(mh);
-#endif
-			select(0, NULL, NULL, NULL, &wait170);
-		}
-	}
-#endif
+	if(!param.smooth && param.usebuffer) buffer_drain();
 	if(param.verbose) print_stat(mh,0,xfermem_get_usedspace(buffermem)); 
-#ifdef HAVE_TERMIOS
-	if(param.term_ctrl) term_restore();
-#endif
 
 	if(!param.quiet)
 	{
@@ -961,53 +1043,25 @@ int main(int argc, char *argv[])
 
 	if (intflag)
 	{
-/* 
- * When HAVE_TERMIOS is defined, there is 'q' to terminate a list of songs, so
- * no pressing need to keep up this first second SIGINT hack that was too
- * often mistaken as a bug. [dk]
- * ThOr: Yep, I deactivated the Ctrl+C hack for active control modes.
- *       Though, some sort of hack remains, still using intflag for track skip.
- */
-#if !defined(WIN32) && !defined(GENERIC)
-#ifdef HAVE_TERMIOS
-	if(!param.term_ctrl)
-#endif
-        {
-		gettimeofday (&now, NULL);
-        	secdiff = (now.tv_sec - start_time.tv_sec) * 1000;
-        	if (now.tv_usec >= start_time.tv_usec)
-          		secdiff += (now.tv_usec - start_time.tv_usec) / 1000;
-        	else
-          		secdiff -= (start_time.tv_usec - now.tv_usec) / 1000;
-        	if (secdiff < 1000)
-					{
-						debug("got the second interrupt: out of here!");
-          		break;
-					}
-					else
-					{
-						debug("It's a track advancement message.");
-						++skip_tracks;
-					}
-	}
-#ifdef HAVE_TERMIOS
-	else if(skip_tracks == 0)
-	{
-		debug("breaking up");
-		break;
-	}
-#endif
-#endif
+		if(!skip_or_die(&start_time)) break;
+
         intflag = FALSE;
 
 #ifndef NOXFERMEM
-        if(param.usebuffer) buffer_resync();
+        if(!param.smooth && param.usebuffer) buffer_resync();
 #endif
       }
     } /* end of loop over input files */
+	/* Ensure we played everything. */
+	if(param.smooth && param.usebuffer)
+	{
+		buffer_drain();
+		buffer_resync();
+	}
 	/* Free up memory used by playlist */    
 	if(!param.remote) free_playlist();
-	safe_exit(0); /* That closes output, too. */
+
+	safe_exit(0); /* That closes output and restores terminal, too. */
 	return 0;
 }
 
@@ -1080,9 +1134,10 @@ static void long_usage(int err)
 	fprintf(o,"\ninput options\n\n");
 	fprintf(o," -k <n> --skip <n>         skip n frames at beginning\n");
 	fprintf(o," -n     --frames <n>       play only <n> frames of every stream\n");
-	fprintf(o," -y     --resync           DISABLES resync on error\n");
+	fprintf(o," -y     --no-resync        DISABLES resync on error (--resync is deprecated)\n");
 	fprintf(o," -p <f> --proxy <f>        set WWW proxy\n");
 	fprintf(o," -u     --auth             set auth values for HTTP access\n");
+	fprintf(o,"        --ignore-mime      ignore HTTP MIME types (content-type)\n");
 	fprintf(o," -@ <f> --list <f>         play songs in playlist <f> (plain list, m3u, pls (shoutcast))\n");
 	fprintf(o," -l <n> --listentry <n>    play nth title in playlist; show whole playlist for n < 0\n");
 	fprintf(o,"        --loop <n>         loop track(s) <n> times, < 0 means infinite loop (not with --random!)\n");
@@ -1102,7 +1157,7 @@ static void long_usage(int err)
 	fprintf(o," -S     --STDOUT           play AND output stream (not implemented yet)\n");
 	fprintf(o," -w <f> --wav <f>          write samples as WAV file in <f> (- is stdout)\n");
 	fprintf(o,"        --au <f>           write samples as Sun AU file in <f> (- is stdout)\n");
-	fprintf(o,"        --cdr <f>          write samples as CDR file in <f> (- is stdout)\n");
+	fprintf(o,"        --cdr <f>          write samples as raw CD audio file in <f> (- is stdout)\n");
 	fprintf(o,"        --reopen           force close/open on audiodevice\n");
 	#ifdef OPT_MULTI
 	fprintf(o,"        --cpu <string>     set cpu optimization\n");
@@ -1127,6 +1182,7 @@ static void long_usage(int err)
 	fprintf(o," -r     --rate             force a specific audio output rate\n");
 	fprintf(o," -2     --2to1             2:1 downsampling\n");
 	fprintf(o," -4     --4to1             4:1 downsampling\n");
+  fprintf(o,"        --pitch <value>    set hardware pitch (speedup/down, 0 is neutral; 0.05 is 5%%)\n");
 	fprintf(o,"        --8bit             force 8 bit output\n");
 	fprintf(o," -d n   --doublespeed n    play only every nth frame\n");
 	fprintf(o," -h n   --halfspeed   n    play every frame n times\n");
@@ -1138,7 +1194,10 @@ static void long_usage(int err)
 	fprintf(o," -o h   --headphones       (aix/hp/sun) output on headphones\n");
 	fprintf(o," -o s   --speaker          (aix/hp/sun) output on speaker\n");
 	fprintf(o," -o l   --lineout          (aix/hp/sun) output to lineout\n");
+#ifndef NOXFERMEM
 	fprintf(o," -b <n> --buffer <n>       set play buffer (\"output cache\")\n");
+	fprintf(o,"        --smooth           keep buffer over track boundaries\n");
+#endif
 
 	fprintf(o,"\nmisc options\n\n");
 	fprintf(o," -t     --test             only decode, no output (benchmark)\n");
